#!/usr/bin/perl -w

use strict;
use warnings;
use Getopt::Long;
use DistroDB::Stems qw//;

my $mode_compare=1;
my $mode_autoimports=0;
my $verbose=0;
my ($help, $opt_dblist);
my $opt_repopath='altlinux/sisyphus';

my @db=@DistroDB::Stems::mapper_stems;

if ($0 =~ /distrodb-check-health-autoimports/) {
    $mode_compare=0;
    $mode_autoimports=1;
}
if ($0 =~ /-(perl)(?:\.pl)?$/) {
    @db=$1;
}
if ($0 =~ /-libs(?:\.pl)?$/) {
    @db=qw/devel-libs static-libs pkg-config gir headers/;
}

GetOptions (
    'repopath=s'  => \$opt_repopath,
    'db=s'  => \$opt_dblist,
    'help'  => \$help,
    'verbose+'  => \$verbose,
);

# todo check $opt_dblist,
if (defined $opt_dblist) {
    @db=();
    @db=split(/[,\s]+/,$opt_dblist) if $opt_dblist;
    print STDERR "new stems: [",join('|',@db),"]\n" if $verbose;
    map {die 'bad stem '.$_ if !$DistroDB::Stems::known_stems{$_}} @db;
}

foreach my $stem (@db) {
    my ($classicdb,$autodb)=({},{});
    my $autodbpattern=$ENV{'HOME'}.'/.cache/distrodb/'.$opt_repopath.'/'.$stem.'/autoimports*.txt';
    my $classicdbpattern=$ENV{'HOME'}.'/.cache/distrodb/'.$opt_repopath.'/'.$stem.'/classic*.txt';
    map {&__load_map_as($_,$autodb)} glob ($autodbpattern);
    map {&__load_map_as($_,$classicdb)} glob ($classicdbpattern);

    if ($mode_compare) {
	foreach (keys(%$autodb)) {
	    if ($classicdb->{$_}) {
		print "error($stem): ".join(',',@{$autodb->{$_}})." conflicts with ".join(',',@{$classicdb->{$_}});
		print " on ",$_ if $verbose;
		print "\n";
	    }
	}
    }

    if ($mode_autoimports) {
	foreach (keys(%$autodb)) {
	    my $autodbref=$autodb->{$_};
	    if (@$autodbref>1) {
		print "problem($stem): ".join(',',@{$autodbref})." overlaps";
		print " on ",$_ if $verbose;
		print "\n";
	    }
	}
    }
}

sub __load_map_as {
    my ($file,$MAP,$MAPBACK)=@_;
    print STDERR "DistroMap: loaded map $file\n" if $verbose and $verbose > 1;
    my $split=qr'\s+';
    $split=qr' *\t+ *' if $file=~/tsv$/;
    $split=qr'|' if $file=~m!/eterbuild/!;
    open (my $fn, '<', $file) || die "can't open $file: $!";
    local $_;
    while (<$fn>) {
	chomp;
	next if /^#/;
	next if /^\s*$/;
	warn "$file: endspace found" if s/\s+$//;
	my @map=grep {$_} split (/$split/, $_);
	my $from=shift @map;
	die "$file: invalid map $_" unless @map;
	$MAP->{$from}=[sort {$a cmp $b} @map];
	$MAPBACK->{$map[0]}=[$from] if $MAPBACK and (scalar @map == 1);
    }
    close ($fn);
}
