#!/usr/bin/perl -w

use strict;
use warnings;
use DistroMap;
use Data::Dumper;
use Getopt::Long;
$Data::Dumper::Indent=1;

$DistroMap::default_originrepo='fedora';
$DistroMap::default_originbranch='default';
$DistroMap::default_destrepo='altlinux';
$DistroMap::default_destbranch='sisyphus';

my $verbose=0;
my %overlaps;

GetOptions (
    @DistroMap::LONGOPT,
    "verbose"  => \$verbose,
);

my $distromap=DistroMap->new();
my $origin=$distromap->origin();
my $dest=$distromap->dest();

my $bin2sourceorigin=$origin->_get_db('sourcename');
foreach my $db (qw/
bin         gir      path  pkg-config  python2  unique_data.raw
devel-libs  headers  perl  provides    python3
/) {
    my $origindb=$origin->_get_db($db);
    my $destdb=$dest->_get_db($db);
    &check_intersection($origindb,$destdb,$db);
}

if (not $verbose) {
    foreach my $originer (keys(%overlaps)) {
	print "$originer\n";
    }
} else {
    print Dumper(\%overlaps);
#    foreach my $originer (keys(%overlaps)) {
#	print "$originer:\n";
#	foreach my $binpkg (keys(%{$overlaps{$originer}})) {
#	    print "\t",$destiner,":\n";

#	foreach my $destiner (keys(%{$overlaps{$originer}})) {
#	    print "\t",$destiner,":\n";
#	}
#	}
#    }
}

sub check_intersection {
    my ($origindb,$destdb,$db)=@_;
    foreach my $key (keys(%$origindb)) {
	my $intersection=$destdb->{$key};
	if ($intersection) {
	    &add_overlap($origindb->{$key}->[0], $intersection->[0], $db, $key);
	}
    }
}

sub add_overlap {
    my ($originpkg, $destpkg, $db, $key)=@_;
    my $originsrc=$bin2sourceorigin->{$originpkg}->[0];
    die "nothing for $originpkg" if not defined $originsrc;
    my $srcentry=$overlaps{$originsrc};
    unless ($srcentry) {
	$srcentry={};
	$overlaps{$originsrc}=$srcentry;
    }
    my $pkgentry=$srcentry->{$originpkg};
    unless ($pkgentry) {
	$pkgentry={};
	$srcentry->{$originpkg}=$pkgentry;
    }
    my $crossentry=$pkgentry->{$destpkg};
    unless ($crossentry) {
	$crossentry=[];
	$pkgentry->{$destpkg}=$crossentry;
    }
    push @$crossentry, "$db: $key";
}
