#!/usr/bin/perl

use strict;
use warnings;
use Carp;
use Getopt::Long;
use File::Basename;
use DistroMap;
use DistroDBTools;

# TODO: add contents_index based unique files mapping based criteria
# (like, mod %bin%, mod usr/share (not doc) if unique-mapping is dence enough (> 50% in the area)?
# to fix likes of broken mapping
# wesnoth-data    python-module-wesnoth

my $ORIGIN;
my $DEST='altlinux/sisyphus';
my $skip_id_map=1;
my $verbose=0;
my $help;

GetOptions (
    'origin=s'  => \$ORIGIN,
    'dest=s'  => \$DEST,
    'help'  => \$help,
    'verbose+'  => \$verbose,
);

if ($help or !$ORIGIN) {
	_usage();
	exit(0);
}

$DistroDBTools::verbose=$verbose;

my $origin={};
my $dest={};

#foreach my $stem (qw/bin path/) {
#    $origin->{$stem}={} unless $origin->{$stem};
#    $dest->{$stem}={} unless $dest->{$stem};
#    &DistroDBTools::read_distrodb_one2one($dest->{$stem}, $DEST, $stem);
#    &DistroDBTools::read_distrodb_one2one($origin->{$stem}, $ORIGIN, $stem);
#}
foreach my $stem (qw/bin path/) {
    $origin->{$stem}={} unless $origin->{$stem};
    $dest->{$stem}={} unless $dest->{$stem};
    &DistroDBTools::read_distrodb_map($dest->{$stem}, $DEST, $stem);
    &DistroDBTools::read_distrodb_map($origin->{$stem}, $ORIGIN, $stem);
}

($DistroMap::default_originrepo, $DistroMap::default_originbranch) = split ('/', $ORIGIN);
($DistroMap::default_destrepo, $DistroMap::default_destbranch) = split ('/', $DEST);
my $distromap=DistroMap->new();

my $origin2dest={};
my $originpath=$origin->{'path'};
my $originbin=$origin->{'bin'};
my $destpath=$dest->{'path'};
my $destbin=$dest->{'bin'};

my $destpath2bin={};
my $destbinandpkgflag={};
my $destpathandpkgflag={};
foreach my $dbin (keys(%$destbin)) {
    my $dpath=basename $dbin;
    $destpath2bin->{$dpath}//=[];
    push @{$destpath2bin->{$dpath}}, $dbin;
    foreach my $dpkg (@{$destbin->{$dbin}}) {
        $destbinandpkgflag->{$dbin}->{$dpkg}=1;
        $destpathandpkgflag->{$dpath}->{$dpkg}=1;
    }
}

#foreach my $dpath (keys(%$destpath2bin)) {
#    print $dpath, ' == ', join(',', @{$destpath2bin->{$dpath}}), "\n";
#}

foreach my $obin (keys(%$originbin)) {
    #print $bin, "\n";
    my @opkg=@{$originbin->{$obin}};
    #my @dpkg=@{$destbin->{$obin}};
    my $path=basename($obin);
    if (@opkg>1) {
        print STDERR "I: $ORIGIN $obin: multiple origin packages ", join(' ', @opkg), "\n";
    } else {
	my @map=$distromap->multimap_binary_origin2dest($opkg[0]);
	if (@map>1) {
	    my $binflag=$destbinandpkgflag->{$obin};
	    my @map1=grep {$binflag->{$_} } @map;
	    my $pathflag=$destpathandpkgflag->{$path};
	    my @map2=grep {$pathflag->{$_} } @map;
	    my @map3=grep {$obin eq $_ } @map; # /usr/bin/luit maps to /usr/bin/luit ....
	    @map=@map2 if @map2;
	    @map=@map1 if @map1;
	    @map=@map3 if @map3;
#	    if ($obin eq '/usr/bin/luit') {
#	        print "$obin: opkg=@opkg, map1=@map1, map2=@map2, map=@map\n";
#	        print "$obin: b=",join(',',keys %$binflag), ", p=", join(',',keys %$pathflag), "\n";
#	    }
	}

        if (@map>1) {
            print STDERR "I: $ORIGIN $obin: multiple dest packages ", join(' ', @map), "\n";
        } elsif (@map==0) {
		#print STDERR "I: $ORIGIN $obin: no dest packages ", join(' ', @map), "\n";
        } elsif (not defined $destpath2bin->{$path}) {
		print STDERR "I: $ORIGIN $obin: no dest binary, has packages ", join(' ', @map), "\n";
        } else {
	    my @dbins=@{$destpath2bin->{$path}};
	    if (@dbins>1) {
		my $altbin=$obin;
		$altbin=~s,^/usr/lib64/qt5/bin,/usr/share/qt5/bin,;
		my $osbin1=$obin;
		$osbin1=~s,^/usr,,;
		my $osbin2='/usr'.$obin;
		my @destsbins=@dbins;
		map {s,^/usr,,} @destsbins;
	        my @dbins1=grep {$_ !~ m!^/usr/lib/kde./bin/! and $_ !~ m!/usr/lib64/qt4/bin/!} @dbins;
	        my @dbins2=grep {$altbin eq $_ or $osbin1 eq $_ or $osbin2 eq $_} @dbins;
	        my @dbins3=grep {$obin eq $_} @dbins;
		@dbins=@dbins1 if @dbins1;
		@dbins=@dbins2 if @dbins2;
		@dbins=@dbins3 if @dbins3;
#		if ($obin eq '/usr/bin/ksysguard') {
#		    print "$obin: dbins=@dbins, dbins1=@dbins1, dbins2=@dbins2, dbins3=@dbins3\n";
#		} 
	    }
	    if (@dbins>1) {
		print STDERR "I: $ORIGIN $obin: multiple dest binaries ", join(' ', @dbins), "\n";
	    } else {
	    #print $obin, ' - ', join(',',@dbins), "\n";
	    	$origin2dest->{$obin}=\@dbins;
	    }
        }
    }
}

&DistroDBTools::sort_print_hasharray($origin2dest,$skip_id_map);

sub generic_translate {
    my ($stem)=@_;
    my $destdb=$dest->{$stem};
    my $origindb=$origin->{$stem};
    foreach my $key (keys(%$origindb)) {
	my $destpkg=$destdb->{$key};
	if ($destpkg) {
	    my $originpkg=$origindb->{$key};
	    &add_distromap($originpkg,$destpkg);
	}
    }
}

sub generic_multival_translate_by_filter {
    my ($stem, $filters)=@_;
    my $destdb=$dest->{$stem};
    my $origindb=$origin->{$stem};
    foreach my $key (keys(%$origindb)) {
	my $destset=$destdb->{$key};
	next unless $destset;
	my $originset=$origindb->{$key};
	if (1==@$destset and 1==@$originset) {
	    &add_distromap($originset->[0],$destset->[0]);
	} else {
	    foreach my $filterpair (@$filters) {
		my $originfiltered=&__filter_using($originset, $filterpair->[0]);
		my $destfiltered=&__filter_using($destset, $filterpair->[1]);
		if (defined $originfiltered and defined $destfiltered) {
		    &add_distromap($originfiltered,$destfiltered);
		}
	    }
	}
    }
}

sub set_distromap {
    my ($from,$to)=@_;
    $origin2dest->{$from}=[$to];
}

sub add_distromap {
    my ($from,$to)=@_;
    &__add_distromap($from,$to);
}

sub __add_distromap {
    my ($from,$to)=@_;
    my $list=$origin2dest->{$from};
    unless ($list) {
	$list=[];
	$origin2dest->{$from}=$list;
    }
    #print STDERR "collision for $from: $origin2dest->{$from} $to\n" if ($old and $old ne $to);
    push @$list, $to unless grep {$_ eq $to} @$list;
}

