#!/usr/bin/perl

use strict;
use warnings;
use Getopt::Long;
use DistroDBTools;

my (%altsrc,%altegg, %altmap, %eggmap, %pypi_known, %altmangled, %pypi_missing);

&DistroDBTools::read_distrodb_map(\%altegg, 'altlinux/sisyphus', 'pyegg');
&DistroDBTools::read_distrodb_one2one(\%altsrc, 'altlinux/sisyphus', 'sourcename');
&read_pypi_list(\%pypi_known,'pypi.txt');


foreach my $eggprov (keys(%altegg)) {
    die if not $eggprov=~/^pythonegg\(\d+\)\((.*)\)$/;
    my $mangledegg=$1;
    foreach my $pkg (@{$altegg{$eggprov}}) {
    	$altmangled{$altsrc{$pkg}}=$mangledegg;
    }
    my $eggref=$pypi_known{$mangledegg};
    unless ($eggref) {
	$pypi_missing{$mangledegg}=1;
	#print STDERR "missing egg $mangledegg\n";
    } elsif (@$eggref>1) {
	print STDERR "Oops: multiple eggs for $mangledegg: ",join(', ',@$eggref),"\n";
    } else {
	foreach my $pkg (@{$altegg{$eggprov}}) {
	    $altmap{$altsrc{$pkg}}=$eggref->[0];
	}
    }
}

foreach my $srcname (keys(%altmap)) {
    my $egg=$altmap{$srcname};
    push @{$eggmap{$egg}}, $srcname;
}

&DistroDBTools::write_map(\%eggmap, 'source/40-distrodb-new.txt');
&DistroDBTools::write_one2one(\%altmangled, 'alt2egg.txt');
&DistroDBTools::write_one2one(\%altmap, 'alt2pypi.txt');
&DistroDBTools::write_flag(\%pypi_missing, 'pypi_missing.txt');

sub read_pypi_list {
    my ($map,$filename)=@_;
    open (my $fn,'<',$filename) or die "can't open $filename: $!";
    while (<$fn>) {
	chomp;
	next if /^\s*#/;
	my @in=split(/\s+/);
	my $egg=shift @in;
	next unless defined $egg;
	my $mangle=&pypi_mangle($egg);
	push @{$map->{$mangle}}, $egg;
    }
    close($fn);
}

sub pypi_mangle {
    my ($egg)=@_;
    $egg=~s/-/_/g;
    return $egg;
}
