package DistroDB::Arrow;

# basic object: relation arrow

use strict;
use warnings;
use Carp;
use Clone 'clone';

use Exporter 'import';
our @EXPORT_OK = qw/
&__arrow_create1
&static_create_raw_arrow
&__aggregate_arrowset_by_pkgset
&__sort_arrowset_by_fingerprint_weight
&__get_arrow_keystring
&__arrowset_debug
&__arrowset_split_by_connected_components
&__arrow_debug
&__arrow_head_pkgs
&__arrow_set_head_pkgset
&__arrow_get_head_pkgset
&__arrow_get_head_pkgset_hashstring
&__arrow_get_fingerprint
/;
#&__arrow_fingerprint_ref


use DistroDB::Pkgset qw/
&__pkgset_create_hashstring
&__pkgset_pkgs
/;
use DistroDB::Fingerprint qw/
&__get_fingerprint_traitstring
&__fingerprint_merge
&__fingerprint_weight
&__fingerprint_is_intersect
/;


sub TODO_new {
    my $class=shift;
    my $self={
	@_,
    };
    bless $self, $class;
}

sub __arrow_create1 {
    my ($fingerprint,$pkg)=@_;
    die 'Oops: undefined fingerprint' if not defined $fingerprint;
    die 'Oops: undefined head' if not defined $pkg;
    die 'Oops: empty fingerprint' if not %$fingerprint;
    my $self={
	-fingerprint => $fingerprint,
	-pkgset => [$pkg],
    };
    #bless $self, __PACKAGE__;
    return $self;
}

sub static_create_raw_arrow {
    my ($fingerprint,$pkgset)=@_;
    die 'Oops: undefined fingerprint' if not defined $fingerprint;
    die 'Oops: undefined head' if not defined $pkgset;
    die 'Oops: empty fingerprint' if not %$fingerprint;
    die 'Oops: empty head:' if not @$pkgset;
    my $self={
	-fingerprint => $fingerprint,
	-pkgset => $pkgset,
    };
    #bless $self, __PACKAGE__;
    return $self;
}

sub __get_arrow_keystring {
    my ($arrow,$joinstring)=@_;
    return &__get_fingerprint_traitstring($arrow->{-fingerprint},$joinstring);
}

sub __arrow_debug {
    my ($arrow,$message)=@_;
    print STDERR 'DEBUG: ';
    print STDERR $message,':' if defined $message;
    print STDERR "\n";
    print STDERR '- pkgset=',join(',',@{$arrow->{-pkgset}}),"\n";
    print STDERR '- ',&__get_arrow_keystring($arrow,"\n".'- '),"\n";
}

sub __arrowset_debug {
    my ($arrowset,$message)=@_;
    print STDERR 'DEBUG: ';
    print STDERR $message,': ' if defined $message;
    print STDERR 'arrows: ',(scalar @$arrowset),"\n";
    for (my $i=0; $i< @$arrowset; $i++) {
	&__arrow_debug($arrowset->[$i], 'arrow '.$i);
    }
}

sub __arrow_get_head_pkgset_hashstring {
    my $arrow=shift;
    my $setkey=$arrow->{-__pkg_set_hash};
    unless ($setkey) {
	$setkey=&__pkgset_create_hashstring($arrow->{-pkgset});
	$arrow->{-__pkg_set_hash}=$setkey;
    }
    return $setkey;
}

sub __aggregate_arrowset_by_pkgset {
    my ($arrowset)=@_;
    return $arrowset if @$arrowset<2;
    my %pkgsethash;
    foreach my $arrow (@$arrowset) {
	my $set=$arrow->{-pkgset};
	next unless $set;
	my $setkey=&__arrow_get_head_pkgset_hashstring($arrow);
	my $aggregated_arrow=$pkgsethash{$setkey};
	unless ($aggregated_arrow) {
	    $aggregated_arrow=clone($arrow);
	    $pkgsethash{$setkey}=$aggregated_arrow;
	} else {
	    $aggregated_arrow->{-fingerprint}=&__fingerprint_merge($aggregated_arrow->{-fingerprint},$arrow->{-fingerprint});
	}
    }
    my @aggregated_arrowset=values(%pkgsethash);
    return \@aggregated_arrowset;
}

# most keys first
sub __sort_arrowset_by_fingerprint_weight {
    my $arrowset=shift;
    my @sorted_arrowset=sort { &__fingerprint_weight($b->{-fingerprint}) <=> &__fingerprint_weight($a->{-fingerprint}) } @$arrowset;
    return \@sorted_arrowset;
}

sub __arrow_head_pkgs {
    return &__pkgset_pkgs($_[0]->{-pkgset});
}

sub __arrow_get_fingerprint {
    return $_[0]->{-fingerprint};
}

sub __arrow_set_fingerprint {
    $_[0]->{-fingerprint}=$_[1];
}

sub __arrow_get_head_pkgset {
    #confess "not an arrow" unless $_[0] and ref $_[0] eq 'HASH';
    return $_[0]->{-pkgset};
}

sub __arrow_set_head_pkgset {
    $_[0]->{-pkgset}=$_[1];
}

sub __arrowset_split_by_connected_components {
    my ($arrowset)=@_;
    my @connected_components;
    while (@$arrowset>0) {
	my $set0=[shift @$arrowset];
	&__arrowset_helper_fill_connected_component($set0,$arrowset);
	push @connected_components, $set0;
    }
    return @connected_components;
}

sub __arrowset_helper_fill_connected_component {
    my ($arrowset1,$arrowset2)=@_;
    return if @$arrowset2==0;
    my @new_arrowset2;
  L1:
    while (@$arrowset2>0) {
	my $arrow2 = pop @$arrowset2;
	foreach my $arrow1 (@$arrowset1) {
	    if (&__arrow_is_fingerprint_intersect($arrow1,$arrow2)) {
		push @$arrowset1, $arrow2;
		next L1;
	    }
	}
	push @new_arrowset2, $arrow2;
    }
    @$arrowset2=@new_arrowset2;
}

sub __arrow_is_fingerprint_intersect {
    return &__fingerprint_is_intersect($_[0]->{-fingerprint},$_[1]->{-fingerprint});
}

1;
