package DistroDB::BinaryDistroMap;

use strict;
use warnings;

use Exporter 'import';
our @EXPORT_OK = qw/
&postprocess_binary_distromap
&add_distromap
$origin2dest
/;

our $origin2dest={};

my ($originprov,$destprov);
my $originreqflag={};

sub initialize {
    my (undef,$origin,$dest)=@_;
    $destprov=$dest->{'provides'};
    $originprov=$origin->{'provides'};
    map {$originreqflag->{$_}=1} keys(%{$origin->{'buildrequires.raw'}}), keys(%{$origin->{'requires.raw'}});
}

sub postprocess_binary_distromap {
    die unless $originprov and $destprov;
    # prov's that are really required are routed to the already calculated deps
    foreach my $key (keys(%$originreqflag)) {
	next if $key =~ m!/!; #/path/
	next if $key =~ m!^lib.*\.so\b!;
	# perl provides a lot of modules
	next if $key =~ m!^perl-!;
	if ($key =~ /\(/) {
	    if ($key =~ /^cmake\(/) {
		# resolve cmake(Foo) provides
	    } elsif ($key =~ /^python3dist\(/) {
		# resolve python3dist(Foo) provides
	    } else {
		#(64bit)...
		next;
	    }
	}
	next if $origin2dest->{$key};
	my $originsetprov=$originprov->{$key};
	next if not $originsetprov or scalar @$originsetprov>1;
	my $originpkg=$originsetprov->[0];
	my $destpkgset=$origin2dest->{$originpkg};
	next unless $destpkgset;
	my $destsetproviders=$destprov->{$key};
	# wrap map provides to collections
	# like eject(util-linux) -> eject  agetty  cfdisk  fdisk   getopt  hwclock look    losetup lsblk   mount util-linux
	# TODO: we need to check not $destpkgset, but $destpkgset+subpackage
	# do we even need to set those maps?
	if (@$destpkgset > 1 and  grep {$_ eq $key} @$destpkgset) {
	    $origin2dest->{$key}=[$key];
	    next;
	}
	# no need to add provides if corresponding provides already exist
	next if $destsetproviders and scalar @$destsetproviders>0 and &are_arrays_intersect($destsetproviders,$destpkgset);
	$origin2dest->{$key}=[@$destpkgset];
    }

    # aliases: not used due to python3 now provides python in fedora
    if (0) {
	foreach my $key (keys(%$origin2dest)) {
	    my $destpkg=$origin2dest->{$key};
	    my $alias=$key;
	    #print STDERR "try: $key\n";
	    # python-foo Provides aliases for python2-foo
	    if ($alias=~s/^python2-/python-/) {
		my $originset=$originprov->{$alias};
		#print STDERR "possible alias: $alias for $key\n" if $originpkg;
		if ($originset and @$originset==1  and $originset->[0] eq $key) {
		    map {&add_distromap($alias,$_)} @$destpkg;
		}
		# TODO lib-devel
	    } elsif (0) {

	    }
	}
    }

my @force_map= # not <...>-common
qw/
automake	automake
autoconf	autoconf
gcc		gcc
gcc-c++	gcc-c++
libtool		libtool
/;
#gcc-gfortran	gcc-fortran
    while (scalar @force_map) {
	&set_distromap(shift @force_map, shift @force_map);
    }

    # long lists of aliases and difference in modules bundled
    &set_distromap('perl','perl');
}

sub set_distromap {
    my ($from,$to)=@_;
    $origin2dest->{$from}=[$to];
}

sub add_distromap {
    my ($from,$to)=@_;
    # python notests hack
    return if $to =~ /^python3?-module-.+-tests$/;
    # devel nodoc hack
    return if $from =~ /-devel$/ and $to =~ /-docs?$/;
    # mageia lib64 hack
    my $libmangled=$from;
    if ($libmangled=~ s/^lib64/lib/) {
	my $oprovset=$originprov->{$libmangled};
	if ($oprovset and @$oprovset==1 and $oprovset->[0] eq $from) {
	    &__add_distromap($libmangled,$to);
	}
    }
    # end mageia lib64 hack
    &__add_distromap($from,$to);
}

sub __add_distromap {
    my ($from,$to)=@_;
    my $list=$origin2dest->{$from};
    unless ($list) {
	$list=[];
	$origin2dest->{$from}=$list;
    }
    #print STDERR "collision for $from: $origin2dest->{$from} $to\n" if ($old and $old ne $to);
    push @$list, $to unless grep {$_ eq $to} @$list;
}

sub are_arrays_intersect {
    my ($arrayref1,$arrayref2)=@_;
    foreach my $element1 (@$arrayref1) {
	foreach my $element2 (@$arrayref2) {
	    return 1 if $element1 eq $element2;
	}
    }
    return 0;
}

sub is_in_array {
    my ($item,$arrayref)=@_;
    foreach my $element (@$arrayref) {
	return 1 if $item eq $element;
    }
    return 0;
}

1;
