package DistroDB::Fingerprint;

# basic object: relation arrow

use strict;
use warnings;

use Clone 'clone';

use Exporter 'import';
our @EXPORT_OK = qw/
&__get_fingerprint_traitstring
&__fingerprint_add
&__fingerprint_merge
&__fingerprint_weight
&__fingerprint_is_empty
&__fingerprint_is_intersect
&__fingerprint_intersect_and_setminus
/;

sub __get_fingerprint_stem_traitstring {
    my $traitset=shift;
    die "Oops: no traits" unless $traitset;
    my @traits=keys(%$traitset);
    my $keycount=scalar @traits;
    if ($keycount==0) {
	die "Oops: no traits 2";
    } elsif ($keycount==1) {
	return $traits[0];
    } elsif ($keycount==2) {
	return $traits[0].', '.$traits[1];
    } elsif ($keycount==3) {
	return $traits[0].', '.$traits[1].', '.$traits[2];
    } else {
	return $traits[0].', '.$traits[1].', '.$traits[2].' and '.($keycount-3).' others';
    }
    die "Can't be";
}

sub __get_fingerprint_traitstring {
    my ($fingerprint,$joinstring)=@_;
    $joinstring//="\n";
    my @str;
    foreach my $stem (keys(%$fingerprint)) {
	push @str, 'stem='.$stem.' traits='.&__get_fingerprint_stem_traitstring($fingerprint->{$stem});
    }
    return join($joinstring, @str);
}

sub __fingerprint_merge {
    my ($fingerprint1,$fingerprint2)=@_;
    my $merge=clone($fingerprint1);
    #foreach my $stem (keys(%$fingerprint1)) {
    #	map {$merge->{$stem}->{$_}=1} keys(%{$fingerprint1->{$stem}});
    #}
    foreach my $stem (keys(%$fingerprint2)) {
	map {$merge->{$stem}->{$_}=1} keys(%{$fingerprint2->{$stem}});
    }
    return $merge;
}

sub __fingerprint_weight {
    my $fingerprint=shift;
    my $weight=0;
    foreach my $stem (keys(%$fingerprint)) {
	$weight+= scalar keys(%{$fingerprint->{$stem}});
    }
    return $weight;
}

sub __fingerprint_is_empty {
    my $fingerprint=shift;
    return 1 if $fingerprint and %$fingerprint;
    return 0;
}

sub __fingerprint_is_intersect {
    my ($fingerprint1,$fingerprint2)=@_;
    foreach my $stem (keys(%$fingerprint2)) {
	my $traitset1=$fingerprint1->{$stem};
	next unless $traitset1;
	my $traitset2=$fingerprint2->{$stem};
	return 1 if grep {$traitset1->{$_} } keys(%$traitset2);
    }
    return 0;
}

sub __fingerprint_intersect_and_setminus {
    my ($fingerprint1,$fingerprint2)=@_;
    my ($fingerprint_common,$fingerprint_1_minus_2)=({},{});
    foreach my $stem (keys(%$fingerprint1)) {
	my $traitset2=$fingerprint2->{$stem};
	my $traitset1=$fingerprint1->{$stem};
	unless ($traitset2) {
	    $fingerprint_1_minus_2->{$stem}=$traitset1;
	} else {
	    my $common={};
	    my $minus12={};
	    foreach my $trait (keys(%$traitset1)) {
		if ($traitset2->{$trait}) {
		    $common->{$trait}=1;
		} else {
		    $minus12->{$trait}=1;
		}
	    }
	    $fingerprint_1_minus_2->{$stem}=$minus12 if %$minus12;
	    $fingerprint_common->{$stem}=$common if %$common;
	}
    }
    return 0;
}


1;
