package DistroDB::Load;

use strict;
use warnings;
use DistroDBTools;

sub load_distrodb_sourceonly {
    my ($topdict, $distroid) = @_;
    foreach my $stem (qw/sourcename/) {
	$topdict->{$stem}=&new_distrodb_dbmap() unless $topdict->{$stem};
	&DistroDBTools::read_distrodb_one2one_with_resolver_table($topdict->{$stem}, $distroid, $stem,
		{
		    qw/unrar unrar/
		});
    }
    foreach my $stem (qw/sourceurl.raw/) {
	$topdict->{$stem}=&new_distrodb_dbmap() unless $topdict->{$stem};
	&DistroDBTools::read_distrodb_one2one($topdict->{$stem}, $distroid, $stem);
    }
}

sub load_distrodb {
    my ($topdict, $distroid, $loaded_stems_ref) = @_;
    &load_distrodb_sourceonly($topdict, $distroid);
    foreach my $stem (qw/buildrequires.raw provides requires.raw/) {
	$topdict->{$stem}=&new_distrodb_dbmap() unless $topdict->{$stem};
	&DistroDBTools::read_distrodb_tsv_map_merge($topdict->{$stem}, $distroid, $stem);
    }
    foreach my $stem (@$loaded_stems_ref) {
	$topdict->{$stem}=&new_distrodb_dbmap() unless $topdict->{$stem};
	&DistroDBTools::read_distrodb_tsv_map_sorted($topdict->{$stem}, $distroid, $stem);
    }
    $topdict->{'same-libs.raw'}=&new_distrodb_dbmap();
    &create_same_libs_from_provides($topdict->{'same-libs.raw'},$topdict->{'provides'});

    # kill known useless duplicates
    &kill_known_useless_duplicates($topdict);
}

sub load_summary_description {
    my ($topdict, $distroid) = @_;
    foreach my $stem (qw/srcversion.raw/) {
	$topdict->{$stem}=&new_distrodb_dbmap() unless $topdict->{$stem};
	&DistroDBTools::read_distrodb_one2one($topdict->{$stem}, $distroid, $stem);
    }
    foreach my $stem (qw/srcsummary.raw srcdescription.raw/) {
	$topdict->{$stem}=&new_distrodb_dbmap() unless $topdict->{$stem};
	&DistroDBTools::read_distrodb_stringval($topdict->{$stem}, $distroid, $stem);
    }
}

sub new_distrodb_dbmap {
    my $new_db={};
    keys(%$new_db)=1024;
    return $new_db;
}

sub kill_known_useless_duplicates {
    my $repo=shift;
    delete $repo->{'python2.raw'}->{'tests/__init__.py'};
    delete $repo->{'python3.raw'}->{'tests/__init__.py'};
    delete $repo->{'python2.raw'}->{'test/__init__.py'};
    delete $repo->{'python3.raw'}->{'test/__init__.py'};
    delete $repo->{'cmake'}->{'Modules'};
    delete $repo->{'cmake'}->{'modules'};
    delete $repo->{'headers'}->{'config.h'};
}

sub create_same_libs_from_provides {
    my ($map,$prov)=@_;
    foreach my $provide (keys(%$prov)) {
	# TODO old -> new (why does not work ?) test!!
	#if ($provide=~/^(lib(?:[^\.]+)\.so\.[\d\.]+)(?:\(\S*\))?\(64bit\)/) {
	if ($provide=~/^(lib(?:[^\.]+)\.so\.[\d\.]+)(?:\(\))?\(64bit\)/) {
	    #my $dep_key=$2;
	    my $lib=$1;
	    $lib=~s,(\.so\.\d+)\.[\.\d]+$,$1,;
	    my $pkgset=$prov->{$provide};
	    my $libref=$map->{$lib};
	    unless ($libref) {
		$map->{$lib}=[@$pkgset];
	    } else {
		push @{$map->{$lib}}, @$pkgset;
	    }
	}
    }
}


1;
