package DistroDB::OriginDestTags;

use strict;
use warnings;

use DistroDB::CLI::SaveState qw/$opt_save_state/;
use DistroDB::CLI::DebugName qw/$debug_obname $debug_oname $debug_dbname $debug_dname/;
#use DistroDBTools;
use DistroDB::Tags qw//;

use Exporter 'import';
our @EXPORT_OK = qw/
$origin_SST_db
$origin_BOT_db
$origin_BMT_db
$origin_BCT_db
$dest_SST_db
$dest_BOT_db
$dest_BMT_db
$dest_BCT_db
/;

my $tags_origin={};
my $tags_dest={};
our $origin_SST_db={};
our $origin_BOT_db={};
our $origin_BMT_db={};
our $origin_BCT_db={};
our $dest_SST_db={};
our $dest_BOT_db={};
our $dest_BMT_db={};
our $dest_BCT_db={};
$tags_origin->{'source'}->{'strict'}=$origin_SST_db;
$tags_origin->{'binary'}->{'optional'}=$origin_BOT_db;
$tags_origin->{'binary'}->{'mandatory'}=$origin_BMT_db;
$tags_origin->{'binary'}->{'conflicting'}=$origin_BCT_db;
$tags_dest->{'source'}->{'strict'}=$dest_SST_db;
$tags_dest->{'binary'}->{'optional'}=$dest_BOT_db;
$tags_dest->{'binary'}->{'mandatory'}=$dest_BMT_db;
$tags_dest->{'binary'}->{'conflicting'}=$dest_BCT_db;

sub initialize {
    my (undef,$origin,$dest)=@_;
    &DistroDB::Tags::read_tags_from_requires($origin->{'sourcename'},$origin->{'requires.raw'},$tags_origin);
    &DistroDB::Tags::read_tags_from_requires($dest->{'sourcename'},$dest->{'requires.raw'},$tags_dest);
    &DistroDB::Tags::read_tags_from_provides($origin->{'sourcename'},$origin->{'provides'},$tags_origin);
    &DistroDB::Tags::read_tags_from_provides($dest->{'sourcename'},$dest->{'provides'},$tags_dest);
    &DistroDB::Tags::read_tags_from_pkgname($origin->{'sourcename'},$tags_origin);
    &DistroDB::Tags::read_tags_from_pkgname($dest->{'sourcename'},$tags_dest);
    &DistroDB::Tags::read_tags_from_python($origin->{'python2.raw'},$origin->{'python3.raw'},$tags_origin);
    &DistroDB::Tags::read_tags_from_python($dest->{'python2.raw'},$dest->{'python3.raw'},$tags_dest);
    # kill strict kde tags for known non-kde.org apps
    foreach my $srcname (qw/libkolab rkward/) {
	delete $origin_SST_db->{$srcname};
	delete $dest_SST_db->{$srcname};
    }
    if ($opt_save_state) {
	mkdir ('state');
	&DistroDBTools::write_set($origin_SST_db,'state/origin_SST.txt');
	&DistroDBTools::write_set($origin_BOT_db,'state/origin_BOT.txt');
	&DistroDBTools::write_set($dest_SST_db,'state/dest_SST.txt');
	&DistroDBTools::write_set($dest_BOT_db,'state/dest_BOT.txt');
	&DistroDBTools::write_set($dest_BMT_db,'state/dest_BMT.txt');
    }

    if ($debug_obname) {
	&DistroDB::Tags::debug_tags_bin_pkgname($tags_origin,$debug_obname);
    }
    if ($debug_dbname) {
	&DistroDB::Tags::debug_tags_bin_pkgname($tags_dest,$debug_dbname);
    }
    if ($debug_oname) {
	my $sourcenameref=$origin->{'sourcename'};
	&DistroDB::Tags::debug_tags_src_pkgname($tags_origin,$debug_oname);
	# TODO: resolve bin names and debug them too
    }
    if ($debug_dname) {
	my $sourcenameref=$dest->{'sourcename'};
	&DistroDB::Tags::debug_tags_src_pkgname($tags_dest,$debug_dname);
	# TODO: resolve bin names and debug them too
    }
}

1;
