package DistroDB::Tags;

use strict;
use warnings;
use Exporter 'import';
our @EXPORT_OK = qw(@optional_binary_tags_order);
use DistroDB::CLI::DebugName qw/$debug_obname $debug_oname/;

our @optional_binary_tags_order=qw/font-100dpi font-75dpi QT KDE GNOME QT3 QT4 QT5 python python3 python2 ruby nodejs ocaml/;

my %strict_source_tags=map {$_=>1} qw/TDE KDE3 KDE4 KDE5/;
my %optional_binary_tags=map {$_=>1} @optional_binary_tags_order;

sub read_tags_from_pkgname {
    my ($sourcenamemap,$tagsflagdb)=@_;
    $tagsflagdb->{'binary'}->{'optional'}//={};
    #$tagsflagdb->{'binary'}->{'mandatory'}//={};
    #$tagsflagdb->{'binary'}->{'conflicting'}//={};
    my $tagsBOT=$tagsflagdb->{'binary'}->{'optional'};
    #my $tagsBMT=$tagsflagdb->{'binary'}->{'mandatory'};
    #my $tagsBCT=$tagsflagdb->{'binary'}->{'conflicting'};
    foreach my $pkgname (keys(%$sourcenamemap)) {
	if ($pkgname=~/\bfont/) {
	    if ($pkgname=~/(100|75)(?:DPI|dpi)/) {
		my $tag='font-'.$1.'dpi';
		&__set_T($tagsBOT,$pkgname,$tag);
	    } elsif ($pkgname=~/(iso8859-\d+)/) {
		my $tag='font-encoding-'.$1;
		&__set_T($tagsBOT,$pkgname,$tag);
	    }
	# set tags from python db, not (only) from pkg names
	# } elsif ($pkgname=~/^[ib]python3(?:$|-)/) {
	#     &__set_T($tagsBOT,$pkgname,'python3');
	#     &__set_T($tagsBCT,$pkgname,'python3');
	# } elsif ($pkgname=~/^[ib]python(?:$|-)/) {
	#     &__set_T($tagsBOT,$pkgname,'python2');
	#     &__set_T($tagsBCT,$pkgname,'python2');
	# } elsif ($pkgname=~/^python3-/) {
	#     &__set_T($tagsBOT,$pkgname,'python3');
	#     &__set_T($tagsBCT,$pkgname,'python3');
	# } elsif ($pkgname=~/^python2?-/) {
	#     &__set_T($tagsBOT,$pkgname,'python2');
	#     &__set_T($tagsBCT,$pkgname,'python2');
	} elsif ($pkgname=~/^(?:ruby|rubygem|gem)-/ or $pkgname=~/-(?:ruby|rubygem|gem)$/) {
	    &__set_T($tagsBOT,$pkgname,'ruby');
	} elsif ($pkgname=~/^(?:node|nodejs|task-nodejs)-/) {
	    &__set_T($tagsBOT,$pkgname,'nodejs');
	} elsif ($pkgname=~/^(?:ocaml)-/) {
	    &__set_T($tagsBOT,$pkgname,'ocaml');
	#} elsif ($pkgname=~/^(php5)/) {
	#    &__set_T($tagsBMT,$pkgname,'php5');
	}
    }
}

sub __set_T {
    #my ($tagsBT,$pkgname,$tag)=@_;
    $_[0]->{$_[1]}->{$_[2]}=1;
}

sub read_tags_from_provides {
    my ($sourcenameref,$prov,$tagsflag)=@_;
    $tagsflag->{'binary'}->{'optional'}//={};
    $tagsflag->{'source'}->{'strict'}//={};
    my $tagsBOT=$tagsflag->{'binary'}->{'optional'};
    my $tagsSST=$tagsflag->{'source'}->{'strict'};
    foreach my $dep (keys(%$prov)) {
	# TODO old -> new
	#if ($dep=~/^(lib[^\.]+\.so\.\d+)[\d\.]*(?:\(\S*\))?\(64bit\)/) {
	if ($dep=~/^(lib[^\.]+\.so\.\d+)[\d\.]*(?:\(\))?\(64bit\)/) {
	    my $shortlib=$1;
	    my @tags=&_lib2tag($shortlib);
	    my @SST=grep {$strict_source_tags{$_}} @tags;
	    my @OBT=grep {$optional_binary_tags{$_}} @tags;
	    my $pkgset=$prov->{$dep};
	    &__set_source_tags($tagsSST,$pkgset,\@SST,$sourcenameref) if @SST;
	    &__set_binary_tags($tagsBOT,$pkgset,\@OBT,$sourcenameref) if @OBT;
	}
    }
}

sub read_tags_from_requires {
    my ($sourcenameref,$req,$tagsflag)=@_;
    $tagsflag->{'binary'}->{'optional'}//={};
    $tagsflag->{'source'}->{'strict'}//={};
    my $tagsBOT=$tagsflag->{'binary'}->{'optional'};
    my $tagsSST=$tagsflag->{'source'}->{'strict'};
    foreach my $dep (keys(%$req)) {
	# TODO old -> new
	#if ($dep=~/^(lib[^\.]+\.so\.\d+)[\d\.]*(?:\(\S*\))?\(64bit\)/) {
	if ($dep=~/^(lib[^\.]+\.so\.\d+)[\d\.]*(?:\(\))?\(64bit\)/) {
	    my $shortlib=$1;
	    my @tags=&_lib2tag($shortlib);
	    my @SST=grep {$strict_source_tags{$_}} @tags;
	    my @OBT=grep {$optional_binary_tags{$_}} @tags;
	    my $pkgset=$req->{$dep};
	    &__set_source_tags($tagsSST,$pkgset,\@SST,$sourcenameref) if @SST;
	    &__set_binary_tags($tagsBOT,$pkgset,\@OBT,$sourcenameref) if @OBT;
	}
    }
}

sub __set_source_tags {
    my ($tagsflag,$pkgset,$tagset,$sourcenameref)=@_;
    foreach my $pkg (@$pkgset) {
	my $srcname=$sourcenameref->{$pkg};
	foreach my $tag (@$tagset) {
	    $tagsflag->{$srcname}->{$tag}=1;
	}
	if ($debug_obname and $debug_obname eq $pkg or $debug_oname and $debug_oname eq $srcname) {
	    print STDERR "DEBUG:DistroDB::Tags: ",$pkg,' set source tags ',join(',',@$tagset),"\n";
	}
    }
}

sub __set_binary_tags {
    my ($tagsflag,$pkgset,$tagset,$sourcenameref)=@_;
    my $srcname;
    foreach my $pkg (@$pkgset) {
	foreach my $tag (@$tagset) {
	    $tagsflag->{$pkg}->{$tag}=1;
	}
	if ($debug_obname and $debug_obname eq $pkg
	    or $debug_oname and $srcname=$sourcenameref->{$pkg} and $srcname eq $debug_oname
	    ) {
	    print STDERR 'DEBUG:DistroDB::Tags: ',$pkg,' set binary tags ',join(',',@$tagset),"\n";
	}
    }
}

sub _lib2tag {
    my $lib=shift;
    if ($lib=~/^lib(?:kdecore|kdeui|kdefx|kio)\.so\.4/) {
	# KDE3
	#libkdecore.so.4
	#libkdeui.so.4
	#libkio.so.4
	return 'KDE3', 'KDE';
    } elsif ($lib=~/^libtde(?:core|ui|fx|io)\.so\./) {
	# TDE
	#libtdecore.so.14
	#libtdefx.so.14
	#libtdeui.so.14
	#	$lib=~/^libtqt(?:|-mt)\.so\./
	#libtqt-mt.so.3
	#libtqt.so.4
	return 'TDE', 'KDE';
    } elsif ($lib=~/^lib(?:kdeclarative|kdecore|kdeui|kdefx|kio)\.so\.5/ or
	     $lib=~/^libkde3support.so.4/ or
	     $lib=~/^libplasma.so.3/ or
	     $lib=~/^libphonon.so.4/
	) {
	# KDE4
	#libkdeclarative.so.5
	#libkdecore.so.5
	#libkio.so.5
	#libplasma.so.3
	return 'KDE4', 'KDE';
    } elsif ($lib=~/^libKF5.*\.so\./) {
	#libKF5.*\.so
	return 'KDE5', 'KDE';
    } elsif ($lib=~/^libqt(?:|-mt)\.so\./) {
	#return 'QT3', 'QT';
	return 'QT';
    } elsif ($lib=~/^libtqt(?:|-mt)\.so\./) {
	#return 'TQT', 'QT';
	return 'QT';
    } elsif ($lib=~/^libQt.*\.so\.4/) {
	#libQtCore.so.4
	#libQt.*\.so.4
	#return 'QT4', 'QT';
	return 'QT';
    } elsif ($lib=~/^libQt5.*\.so\.5/) {
	#libQt5Core.so.5
	#libQt5.*\.so.5
	#return 'QT5', 'QT';
	return 'QT';
    }
    return;
}

sub read_tags_from_python {
    my ($py2map,$py3map,$tagsflagdb)=@_;
    $tagsflagdb->{'binary'}->{'optional'}//={};
    $tagsflagdb->{'binary'}->{'conflicting'}//={};
    my $tagsBOT=$tagsflagdb->{'binary'}->{'optional'};
    my $tagsBCT=$tagsflagdb->{'binary'}->{'conflicting'};
    my (%py2name,%py3name);
    while (my ($key, $value) = each %$py2map) {
	map {$py2name{$_}=1} @$value;
    }
    while (my ($key, $value) = each %$py3map) {
	map {$py3name{$_}=1} @$value;
    }
    foreach my $pkgname (keys(%py2name)) {
	if ($pkgname=~/^[ib]python2?(?:$|-)/) {
	    &__set_T($tagsBOT,$pkgname,'python');
	    &__set_T($tagsBOT,$pkgname,'python2');
	    &__set_T($tagsBCT,$pkgname,'python2');
	} elsif ($pkgname=~/-python2$/) {
	    &__set_T($tagsBOT,$pkgname,'python');
	    &__set_T($tagsBOT,$pkgname,'python2');
	    &__set_T($tagsBCT,$pkgname,'python2');
	} elsif ($pkgname=~/^python2?-/) {
	    &__set_T($tagsBOT,$pkgname,'python');
	    &__set_T($tagsBOT,$pkgname,'python2');
	    &__set_T($tagsBCT,$pkgname,'python2');
	}
    }
    foreach my $pkgname (keys(%py3name)) {
	if ($pkgname=~/^[ib]python3?(?:$|-)/) {
	    &__set_T($tagsBOT,$pkgname,'python');
	    &__set_T($tagsBOT,$pkgname,'python3');
	    &__set_T($tagsBCT,$pkgname,'python3');
	} elsif ($pkgname=~/^python3-/) {
	    &__set_T($tagsBOT,$pkgname,'python');
	    &__set_T($tagsBOT,$pkgname,'python3');
	    &__set_T($tagsBCT,$pkgname,'python3');
	} elsif ($pkgname=~/-python3$/) {
	    &__set_T($tagsBOT,$pkgname,'python');
	    &__set_T($tagsBOT,$pkgname,'python3');
	    &__set_T($tagsBCT,$pkgname,'python3');
	}
    }
}

sub debug_tags_bin_pkgname {
    my ($tagsflagdb, $pkgname)=@_;
    my $tagsBOT=$tagsflagdb->{'binary'}->{'optional'};
    my $tagsBMT=$tagsflagdb->{'binary'}->{'mandatory'};
    my $tagsBCT=$tagsflagdb->{'binary'}->{'conflicting'};
    my $BOTref=$tagsBOT->{$pkgname} if $tagsBOT;
    my $BMTref=$tagsBMT->{$pkgname} if $tagsBMT;
    my $BCTref=$tagsBCT->{$pkgname} if $tagsBCT;
    print STDERR 'DEBUG:DistroDB::Tags: ',$pkgname,' got BO tags ',join(',',keys(%$BOTref)),"\n" if $BOTref;
    print STDERR 'DEBUG:DistroDB::Tags: ',$pkgname,' got BM tags ',join(',',keys(%$BMTref)),"\n" if $BMTref;
    print STDERR 'DEBUG:DistroDB::Tags: ',$pkgname,' got BC tags ',join(',',keys(%$BCTref)),"\n" if $BCTref;
}

sub debug_tags_src_pkgname {
    my ($tagsflagdb, $pkgname)=@_;
    my $tagsSST=$tagsflagdb->{'source'}->{'strict'};
    my $SSTref=$tagsSST->{$pkgname} if $tagsSST;
    print STDERR 'DEBUG:DistroDB::Tags: ',$pkgname,' got SS tags ',join(',',keys(%$SSTref)),"\n" if $SSTref;
}

1;
